Index: channels/chan_sip.c
===================================================================
--- channels/chan_sip.c	(revision 368556)
+++ channels/chan_sip.c	(working copy)
@@ -9549,6 +9549,17 @@
 		}
 	}
 
+	/* Asterisk can't currently handle streams with differing hold states */
+	if (portno != -1 && vportno != -1 && sendonly != vsendonly) {
+		int tmp_so = (sendonly > -1 ? sendonly : 0);
+		int tmp_vso = (vsendonly > -1 ? vsendonly : 0);
+		if (tmp_so != tmp_vso) {
+			ast_log(LOG_WARNING, "Video stream disabled due to mismatched stream hold states\n");
+			vportno = -1;
+			p->novideo = TRUE;
+		}
+	}
+
 	/* Sanity checks */
 	if (!sa && !vsa && !tsa && !isa) {
 		ast_log(LOG_WARNING, "Insufficient information in SDP (c=)...\n");
@@ -9599,6 +9610,11 @@
 		change_t38_state(p, T38_DISABLED);
 	}
 
+	/* Allow the video stream to control the hold state if there is no audio stream */
+	if (portno == -1 && vportno != -1) {
+		sendonly = vsendonly;
+	}
+
 	/* Now gather all of the codecs that we are asked for: */
 	ast_rtp_codecs_payload_formats(&newaudiortp, peercapability, &peernoncodeccapability);
 	ast_rtp_codecs_payload_formats(&newvideortp, vpeercapability, &vpeernoncodeccapability);
@@ -9830,11 +9846,11 @@
 		ast_set_write_format(p->owner, ast_channel_writeformat(p->owner));
 	}
 
-	if (ast_test_flag(&p->flags[1], SIP_PAGE2_CALL_ONHOLD) && (!ast_sockaddr_isnull(sa) || !ast_sockaddr_isnull(vsa) || !ast_sockaddr_isnull(tsa) || !ast_sockaddr_isnull(isa)) && (!sendonly || sendonly == -1)) {
+	if (ast_test_flag(&p->flags[1], SIP_PAGE2_CALL_ONHOLD) && (!ast_sockaddr_isnull(sa) || !ast_sockaddr_isnull(vsa) || !ast_sockaddr_isnull(tsa) || !ast_sockaddr_isnull(isa)) && (!sendonly || sendonly == -1 || sendonly == 3)) {
 		ast_queue_control(p->owner, AST_CONTROL_UNHOLD);
 		/* Activate a re-invite */
 		ast_queue_frame(p->owner, &ast_null_frame);
-		change_hold_state(p, req, FALSE, sendonly);
+		change_hold_state(p, req, FALSE, (sendonly != 3 ? sendonly : -1));
 	} else if ((sockaddr_is_null_or_any(sa) && sockaddr_is_null_or_any(vsa) && sockaddr_is_null_or_any(tsa) && sockaddr_is_null_or_any(isa)) || (sendonly && sendonly != -1)) {
 		ast_queue_control_data(p->owner, AST_CONTROL_HOLD,
 				       S_OR(p->mohsuggest, NULL),
@@ -9974,6 +9990,12 @@
 		if (*sendonly == -1)
 			*sendonly = 2;
 		found = TRUE;
+	} else if (!strcasecmp(a, "recvonly")) {
+		if (*sendonly == -1) {
+			ast_log(LOG_WARNING, "a=recvonly is not supported\n");
+			*sendonly = 3;
+		}
+		found = TRUE;
 	}  else if (!strcasecmp(a, "sendrecv")) {
 		if (*sendonly == -1)
 			*sendonly = 0;
